@@ -52,23 +52,28 @@
 }
 
 
-.index <- function(tt, var_tt, base, level=NULL, sig2=NULL, alt=FALSE) {
+.index <- function(tt, var_tt, base, level=NULL, sig2=NULL, formal=TRUE) {
 
   nbase <- length(base)
 
   if (nbase==1) {
-    tau <- tt / tt[base]
+    scale <- 1 / tt[base]
   } else {
-    tau <- tt / mean(tt[base])
+    scale <- 1 / mean(tt[base])
   }
+  tau <- tt * scale
 
   J <- length(tt)
   var_tau <- numeric(J)
   d <- matrix(0, nbase+1, 1)
   for (j in 1:J) {
 
-    if (nbase==1 && alt==TRUE) {
-      var_tau[j] <- var_tt[j,j] / tt[base]^2
+    if (nbase==1 && !formal) {
+      # compute scaled standard error
+      #var_tau[j] <- var_tt[j,j] / tt[base]^2
+      SE <- sqrt(var_tt[j,j])
+      SE_scaled  <- SE * scale
+      var_tau[j] <- SE_scaled^2
       next
     }
 
@@ -91,6 +96,7 @@
     out$lo_tau <- tau - mul$lo * sqrt(var_tau)
     out$hi_tau <- tau + mul$hi * sqrt(var_tau)
   }
+
   out
 }
 
@@ -99,24 +105,31 @@
 #' Extract time-indices from TRIM output.
 #'
 #' Indices are obtained by dividing the modelled or imputed time totals by a reference value.
-#' Most commonly, the time totals for the first time point are used as reference.
-#' As a result, the index value for this first time point will be 1.0, with a standard error of 0.0 by definition.
-#' Alternatively, a range of time points can be used as reference. In this case, the mean time totals for this range will be used as
-#' reference, and the standard errors will be larger than 0.0.
+#' Most commonly, the time totals for the starting year are used as reference.
+#' As a result, the index value for this year will be 1.0, with a standard error of 0.0 by definition.\cr
+#' Alternatively, a range of years can be used as reference. In this case, the mean time totals for this range will be used as
+#' reference, and the standard errors will be larger than 0.0.\cr
+#' Starting with \code{rtrim} 2.2, an additional method can be selected,
+#' which uses a simpler scaling approach to standard errors of the indices
 #'
 #' @param x an object of class \code{\link{trim}}
-#' @param which \code{[character]} Selector to distinguish between time indices based on the imputed data (default),
+#' @param which (character) Selector to distinguish between time indices based on the imputed data (default),
 #' the fitted model, or both.
-#' @param covars \code{[logical]} Switch to compute indices for covariate categories as well.
-#' @param base \code{[integer|numeric]} One or more base time point, used as as reference for the index.
-#' If just a single number is given, the time total of the correspondong time point will be uses as  reference.
-#' If a range of numbers is given, the average of the corresponding time totals will be used as reference.
-#' The base time points can be given in the interval 1...J, or,
-#' if the time points are proper years, say year1...yearn, the base year can be given.
-#' So, if the data range 2000...2016, \code{base=2} and \code{base=2001} are equivalent.
-#' @param level \code{[numeric]} the confidence interval required.
+#' @param covars (logical) Switch to compute indices for covariate categories as well.
+#' @param base (integer or numeric) One or more years, used as as reference for the index.
+#' If just a single year is given, the time total of the corresponding year will be uses as a reference.
+#' If a range of years is given, the average of the corresponding time totals will be used as reference.\cr
+#' Alternatively, the reference year(s) can be identified using their rank number,
+#' i.e. \code{base=1} always refers to the starting year, \code{base=2} to the second year, etc.
+#' @param level (numeric) the confidence interval required.
 #' Must be in the range 0 to 1. A value of 0.95 results in 95\% confidence intervals.
 #' The default value of NULL results in no confidence interval to be computed.
+#' @param method (character) Method selector.
+#' Options are \code{"formal"} (default) to use a formal computation of standard errors,
+#' resulting in \eqn{\text{SE}=0} for the reference year,
+#' and \code{"scaled"} to use a simpler approach, based on linear scaling of the time-totals SE.
+#' @param long (logical) Switch to return 'long' output
+#' (default is 'wide', as in rtrim versions < 2.2)
 #'
 #' @return A data frame containing indices and their uncertainty expressed as
 #'   standard error. Depending on the chosen output, columns \code{fitted}
@@ -124,9 +137,11 @@
 #'   If \code{covars} is \code{TRUE}, additional indices are computed for the
 #'   individual covariate categories. In this case additional columns
 #'   \code{covariate} and \code{category} are present. The overall indices are
-#'   marked as covariate `Overall' and category 0.
-#'
-#'
+#'   marked as covariate `Overall' and category 0.\cr
+#'   In case \code{long=TRUE} a long table is returned, and a different naming convention is used.
+#'   e.g., imputed/fitted info is in column \code{series},
+#'   and standard error are always in column \code{SE}.
+
 #' @export
 #'
 #' @family analyses
@@ -149,8 +164,9 @@
 #' index(z, base=3)
 #' # Use average of first 5 years as reference for indexing
 #' index(z, base=1:5)
-index <- function(x, which=c("imputed","fitted","both"), covars=FALSE, base=1, level=NULL) {
-  alt <- FALSE # previous: option to just scale by base year TT
+#' # Prevent SE=0 for the reference year
+#' index(z, method="scaled")
+index <- function(x, which=c("imputed","fitted","both"), covars=FALSE, base=1, level=NULL, method=c("formal","scaled"), long=FALSE) {
   stopifnot(inherits(x,"trim"))
 
   # Match base to actual time points
@@ -167,42 +183,93 @@ index <- function(x, which=c("imputed","fitted","both"), covars=FALSE, base=1, l
 
   # Start with overall indices (i.e. ignoring covariate categories, if applicable)
   # Computation and output is user-configurable
-  which <- match.arg(which)
-  if (which=="fitted") {
+  which  <- match.arg(which)
+  method <- match.arg(method)
+  formal <- method=="formal"
+
+  if (which=="fitted") {                # -- FITTED --
     # Call workhorse function to do the actual computation
-    mod <- .index(x$tt_mod, x$var_tt_mod, base, level, x$sig2, alt)
+    mod <- .index(x$tt_mod, x$var_tt_mod, base, level, x$sig2, formal)
     # Store results in a data frame
-    out <- data.frame(time  = x$time.id,
-                      fitted = mod$tau,
-                      se_fit = sqrt(mod$var_tau))
+    if (long==TRUE) {                   # long format
+      out <- data.frame(
+        variable = "index",
+        series   = "fitted",
+        year     = x$time.id,
+        value    = mod$tau,
+        SE       = sqrt(mod$var_tau)
+      )
+    } else {                            # wide format
+      out <- data.frame(
+        time  = x$time.id,
+        fitted = mod$tau,
+        se_fit = sqrt(mod$var_tau))
+    }
+    # Add confidence interval?
     if (!is.null(level)) {
       out$lo <- mod$lo
       out$hi <- mod$hi
     }
-  } else if (which=="imputed") {
+  } else if (which=="imputed") {        # -- IMPUTED --
     # Idem, using the imputed time totals instead
-    imp <- .index(x$tt_imp, x$var_tt_imp, base, level, x$sig2, alt)
-    out = data.frame(time    = x$time.id,
-                     imputed = imp$tau,
-                     se_imp  = sqrt(imp$var_tau))
+    imp <- .index(x$tt_imp, x$var_tt_imp, base, level, x$sig2, formal)
+    if (long==TRUE) {
+      out <- data.frame(
+        variable = "index",
+        series   = "imputed",
+        year     = x$time.id,
+        value    = imp$tau,
+        SE       = sqrt(imp$var_tau)
+      )
+    } else {
+      out = data.frame(time    = x$time.id,
+                       imputed = imp$tau,
+                       se_imp  = sqrt(imp$var_tau))
+    }
     if (!is.null(level)) {
       out$lo <- imp$lo
       out$hi <- imp$hi
     }
-  } else if (which=="both") {
+  }
+  else if (which=="both") {           # -- BOTH --
     # Idem, using both modelled and imputed time totals.
-    if (!is.null(level)) stop("Confidence intervals can only be computed for either imputed or fitted indices, but not for both")
-    mod <- .index(x$tt_mod, x$var_tt_mod, base, alt)
-    imp <- .index(x$tt_imp, x$var_tt_imp, base, alt)
-    out = data.frame(time    = x$time.id,
-                     fitted   = mod$tau,
-                     se_fit  = sqrt(mod$var_tau),
-                     imputed = imp$tau,
-                     se_imp  = sqrt(imp$var_tau))
+    mod <- .index(x$tt_mod, x$var_tt_mod, base, level, x$sig2, formal)
+    imp <- .index(x$tt_imp, x$var_tt_imp, base, level, x$sig2, formal)
+    if (long==TRUE) {
+      out1 <- data.frame(
+        variable = "index",
+        series   = "fitted",
+        year     = x$time.id,
+        value    = mod$tau,
+        SE       = sqrt(mod$var_tau)
+      )
+      out2 <- data.frame(
+        variable = "index",
+        series   = "imputed",
+        year     = x$time.id,
+        value    = imp$tau,
+        SE       = sqrt(imp$var_tau)
+      )
+      if (!is.null(level)) {
+        out1$lo <- mod$lo
+        out1$hi <- mod$hi
+        out2$lo <- imp$lo
+        out2$hi <- imp$hi
+      }
+      out <- rbind(out1, out2)
+    } else { # Wide format
+      if (!is.null(level)) stop("Confidence intervals can only be output for either imputed or fitted indices, but not for both")
+      out = data.frame(time    = x$time.id,
+                       fitted   = mod$tau,
+                       se_fit  = sqrt(mod$var_tau),
+                       imputed = imp$tau,
+                       se_imp  = sqrt(imp$var_tau))
+    }
   } else stop("Can't happen") # because other cases are catched by match.arg()
 
   # Add indices for covariate categories, if applicable
   if (covars) {
+    # create first, "overall" section by pasting a "left" covar part with the "right" index data block
     out <- cbind(data.frame(covariate="Overall", category="(none)", stringsAsFactors=TRUE), out)
 
     tt <- x$covar_tt
@@ -217,32 +284,53 @@ index <- function(x, which=c("imputed","fitted","both"), covars=FALSE, base=1, l
         ttij <- tti[[j]]
         catname <- as.character(ttij$class) # old code; todo: fix properly by setting a cat name earlier
         catname <- levels(x$covars[[name]])[j]
-        df = data.frame(covariate=ttij$covariate, category=catname, time=x$time.id, stringsAsFactors=TRUE)
+        # construct left block (ensure proper # of rows)
+        n <- length(x$time.id)
+        df <- data.frame(covariate = rep(ttij$covariate, n),
+                         category  = rep(catname, n),
+                         stringsAsFactors=TRUE)
+        if (!long) df$time <- x$time.id
         # Compute model index+variance
         if (which %in% c("fitted","both")) {
-          idx <- .index(ttij$mod, ttij$var_mod, base, level, x$sig2, alt)
-          df2 <- data.frame(fitted=idx$tau, se_fit=sqrt(idx$var_tau))
+          idx <- .index(ttij$mod, ttij$var_mod, base, level, x$sig2, formal)
+          if (long) {
+            df2 <- data.frame(variable="index",series="fitted", year=x$time.id, value=idx$tau, SE=sqrt(idx$var_tau))
+          } else{
+            df2 <- data.frame(fitted=idx$tau, se_fit=sqrt(idx$var_tau))
+          }
           if (!is.null(level)) {
             df2$lo <- idx$lo
             df2$hi <- idx$hi
           }
-          df <- cbind(df, df2)
+          if (long) {
+            out <- rbind(out, cbind(df, df2))
+          } else {
+            df <- cbind(df, df2)
+          }
         }
         # Idem for imputed index + variance
         if (which %in% c("imputed","both")) {
-          idx <- .index(ttij$imp, ttij$var_imp, base, level, x$sig2, alt)
-          df2 <- data.frame(imputed=idx$tau, se_imp=sqrt(idx$var_tau))
+          idx <- .index(ttij$imp, ttij$var_imp, base, level, x$sig2, formal)
+          if (long) {
+            df2 <- data.frame(variable="index",series="imputed", year=x$time.id, value=idx$tau, SE=sqrt(idx$var_tau))
+          } else {
+            df2 <- data.frame(imputed=idx$tau, se_imp=sqrt(idx$var_tau))
+          }
           if (!is.null(level)) {
             df2$lo <- idx$lo
             df2$hi <- idx$hi
           }
-          df <- cbind(df, df2)
+          if (long) {
+            out <- rbind(out, cbind(df, df2))
+          } else {
+            df <- cbind(df, df2)
+          }
         }
-        out <- rbind(out, df)
+        if (!long) out <- rbind(out, df)
       }
     }
   }
-  class(out) <- c("trim.index","data.frame")
+  class(out) <- c("trim.index", class(out)) # prepend new class type
   out
 }
 
@@ -320,7 +408,7 @@ plot.trim.index <- function(x, ..., names=NULL, covar="auto", xlab="auto", ylab=
         nidx <- nidx + 1
         zz[[nidx]] <- item
         keep[i] <- FALSE # additional index data sets are removed from the ellipsis argument
-      } else if (class(item)=="character") {
+      } else if (inherits(item, "character")) {
         # todo: check if this arguments immediately follows an index argument
         attr(zz[[nidx]], "tag") <- item
         keep[i] <- FALSE
